<?php

/**
 *	Clase para el inicio de sesión con la API de Google
 *	@Linkin
 */

require_once 'includes/libs/google-api/vendor/autoload.php';

class GoogleAuth implements externalAuth
{
	private $GoogleObj = NULL;
	private $AccountData = NULL;

	public function __construct()
	{	
		if($this->isActiveMode())
		{
			$this->GoogleObj	= new Google\Client();
			$this->GoogleObj->setClientId(Config::get()->google_apikey);
			$this->GoogleObj->setClientSecret(Config::get()->google_skey);
			$this->GoogleObj->setRedirectUri(HTTP_PATH.'index.php?page=ExternalAuth&method=google');
			$this->GoogleObj->addScope(array('email', 'profile'));
		}
	}

	public function isActiveMode()
	{
		return Config::get()->google_on == 1;
	}

	public function isValid()
	{
		$Code = HTTP::_GP('code', '', true);

		if (!empty($Code)) {
			$token = $this->GoogleObj->fetchAccessTokenWithAuthCode($Code);
			if (!isset($token['error'])) {
				$this->GoogleObj->setAccessToken($token['access_token']);
				$this->AccountData = $this->getAccount();
				if($this->AccountData) {
					$LoginData = $this->getLoginData();
					if (empty($LoginData['id'])) {
						$this->register();
					}
					return $this->AccountData;
				}
			} else {
				HTTP::redirectTo('index.php');
			}
		}

		HTTP::sendHeader('Location', $this->GoogleObj->createAuthUrl());
		exit;
	}

	public function getAccount()
	{
		$google_service = new Google_Service_Oauth2($this->GoogleObj);
		return $google_service->userinfo->get();
		
	}

	public function register()
	{
		global $LNG;

		$UserId = self::CheckIfRegister($this->AccountData['email']);
		
		if (!empty($UserId)) {

			$sql = "INSERT INTO %%USERS_AUTH%% SET id = :id, account = :account, mode = :mode;";
			Database::get()->insert($sql, array(
				':id'		=> $UserId,
				':account'	=> $this->AccountData['email'],
				':mode'		=> 'google'
			));

		} else {
			$PassWord 	= md5(uniqid());
			list($UserId, $PlanetId) = PlayerUtil::createPlayer(Universe::current(), $this->AccountData['name'], $PassWord, $this->AccountData['email'], $LNG->getLanguage());

			if (Config::get()->mail_active) {
				require 'includes/classes/Mail.class.php';
				$MailRAW    = $LNG->getTemplate('google_reg');
				$MailContent  = str_replace(array(
			      	'{USERNAME}',
			      	'{PASSWORD}',
			      	'{GAMENAME}'
			    ), array(
			      	$this->AccountData['name'],
			      	$PassWord,
			      	Config::get()->game_name
			    ), 	$MailRAW);

				$subject  = sprintf($LNG['mail_google_subject'], Config::get()->game_name);
    			Mail::send($this->AccountData['email'], $this->AccountData['name'], $subject, $MailContent);
			}
			$sql = "INSERT INTO %%USERS_AUTH%% SET id = :id, account = :account, mode = :mode;";
			Database::get()->insert($sql, array(
				':id'		=> $UserId,
				':account'	=> $this->AccountData['email'],
				':mode'		=> 'google'
			));
		}
	}

	private static function CheckIfRegister($Email)
	{
		$sql = "SELECT id FROM %%USERS%% WHERE email_2 = :mailAddress;";
		$UserExist = database::get()->selectSingle($sql, array(
			':mailAddress'	=> $Email
		), 'id');

		return $UserExist;
	}

	public function getLoginData()
	{
		$sql	= 'SELECT user.id, id_planet
		FROM %%USERS_AUTH%% auth
		INNER JOIN %%USERS%% user ON auth.id = user.id AND user.universe = :universe
		WHERE auth.account = :accountId AND mode = :mode;';

		return Database::get()->selectSingle($sql, array(
			':mode'			=> 'google',
			':accountId'	=> $this->AccountData['email'],
			':universe'		=> Universe::current()
		));
	}

	public function getAccountData()
	{
		return $this->AccountData;
	}
}